@@ -10,4 +10,4 @@
  *
  * The TYPO3 project - inspiring people to share!
  */
-export var TreeNodeCommandEnum;!function(e){e.COPY="copy",e.EDIT="edit",e.MOVE="move",e.DELETE="delete",e.NEW="new"}(TreeNodeCommandEnum||(TreeNodeCommandEnum={}));export var TreeNodePositionEnum;!function(e){e.INSIDE="inside",e.BEFORE="before",e.AFTER="after"}(TreeNodePositionEnum||(TreeNodePositionEnum={}));
\ No newline at end of file
+var t;(function(E){E.COPY="copy",E.EDIT="edit",E.MOVE="move",E.DELETE="delete",E.NEW="new"})(t||(t={}));var f;(function(E){E.INSIDE="inside",E.BEFORE="before",E.AFTER="after"})(f||(f={}));export{t as TreeNodeCommandEnum,f as TreeNodePositionEnum};
