@@ -33,7 +33,10 @@ use TYPO3\CMS\Core\Utility\MathUtility;
  */
 class TcaMigration
 {
-    /** Accumulate migration messages */
+    /**
+     * Accumulate migration messages; reset on each migration() call so
+     * that this service can be used in DI despite carrying state.
+     */
     protected array $messages = [];
 
     /**
@@ -48,6 +51,7 @@ class TcaMigration
      */
     public function migrate(array $tca): array
     {
+        $this->messages = [];
         $this->validateTcaType($tca);
         $this->deprecateSubTypes($tca);
 
@@ -95,6 +99,7 @@ class TcaMigration
 
     /**
      * Get messages of migrated fields. Can be used for deprecation messages after migrate() was called.
+     * Messages will be reset on each `migrate()` execution.
      *
      * @return array Migration messages
      */
@@ -1389,6 +1394,9 @@ class TcaMigration
                 ) {
                     $hasLegacyItemConfiguration = false;
                     $items = $fieldConfig['config']['items'];
+                    if (is_string($items)) {
+                        continue;
+                    }
                     foreach ($items as $key => $item) {
                         if (!is_array($item)) {
                             continue;
