@@ -33,9 +33,12 @@ interface FieldTypeInterface
     public function supportsAccessControl(): bool;
     public function isRequired(): bool;
     public function isNullable(): bool;
+    public function isSearchable(): bool;
     public function getDisplayConditions(): array|string;
     public function getDefaultValue(): mixed;
+    public function hasDefaultValue(): bool;
     public function getTranslationBehaviour(): FieldTranslationBehaviour;
     public function getConfiguration(): array;
+    public function getSoftReferenceKeys(): array|false;
     public static function __set_state(array $state): FieldTypeInterface;
 }
