@@ -27,8 +27,33 @@ final readonly class UuidFieldType extends AbstractFieldType
         return 'uuid';
     }
 
+    public function isSearchable(): true
+    {
+        return true;
+    }
+
     public function getVersion(): int
     {
         return in_array($this->configuration['version'] ?? 0, [4, 6, 7], true) ? $this->configuration['version'] : 4;
     }
+
+    public function isNullable(): false
+    {
+        return false;
+    }
+
+    public function getDefaultValue(): string
+    {
+        return '';
+    }
+
+    public function hasDefaultValue(): true
+    {
+        return true;
+    }
+
+    public function getSoftReferenceKeys(): false
+    {
+        return false;
+    }
 }
