@@ -20,9 +20,12 @@ namespace TYPO3\CMS\Core\Site\Set;
 use Symfony\Component\DependencyInjection\Attribute\Autoconfigure;
 use Symfony\Component\Yaml\Exception\ParseException;
 use Symfony\Component\Yaml\Yaml;
+use TYPO3\CMS\Core\Configuration\Loader\Exception\YamlParseException;
+use TYPO3\CMS\Core\Configuration\Loader\YamlFileLoader;
 use TYPO3\CMS\Core\Settings\CategoryDefinition;
+use TYPO3\CMS\Core\Settings\InvalidSettingDefinitionException;
 use TYPO3\CMS\Core\Settings\SettingDefinition;
-use TYPO3\CMS\Core\Settings\SettingsTypeRegistry;
+use TYPO3\CMS\Core\Settings\SettingDefinitionValidation;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
@@ -31,12 +34,12 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
 #[Autoconfigure(public: true)]
 class YamlSetDefinitionProvider
 {
-    public function __construct(
-        protected readonly SettingsTypeRegistry $settingsTypeRegistry
-    ) {}
-
     /** @var array<string, SetDefinition> */
     protected array $sets = [];
+    public function __construct(
+        protected readonly SettingDefinitionValidation $settingDefinitionValidation,
+        protected readonly YamlFileLoader $yamlFileLoader,
+    ) {}
 
     /**
      * @return array<string, SetDefinition>
@@ -95,16 +98,14 @@ class YamlSetDefinitionProvider
         $settingsFile = $path . '/settings.yaml';
         if (is_file($settingsFile)) {
             try {
-                $settings = Yaml::parseFile($settingsFile);
-            } catch (ParseException $e) {
+                // HEADS UP: YamlFileLoader::PROCESS_PLACEHOLDERS is omitted on purpose and MUST NOT be added.
+                // Site sets are intended to be self-contained and must not rely on implicit
+                // dependencies to global (environment) variables.
+                $settings = $this->yamlFileLoader->load($settingsFile, YamlFileLoader::PROCESS_IMPORTS | YamlFileLoader::ALLOW_EMPTY_FILE);
+            } catch (YamlParseException $e) {
                 $source = $virtualSetPath . basename($settingsFile);
                 throw new InvalidSettingsException('Invalid settings format. Source: ' . $source, 1711024380, $e, $setName);
             }
-            $settings ??= [];
-            if (!is_array($settings)) {
-                $source = $virtualSetPath . basename($settingsFile);
-                throw new InvalidSettingsException('Invalid settings format. Source: ' . $source, 1711024382, null, $setName);
-            }
             $set['settings'] = $settings;
         }
 
@@ -143,12 +144,15 @@ class YamlSetDefinitionProvider
                     $set['name'] ?? ''
                 );
             }
-            if (!$this->settingsTypeRegistry->has($definition->type)) {
-                throw new InvalidSettingsDefinitionsException('Invalid settings type "' . $definition->type . '" for settings definition: ' . json_encode($options), 1732181103, null, $set['name'] ?? '');
-            }
-            $type = $this->settingsTypeRegistry->get($definition->type);
-            if (!$type->validate($definition->default, $definition)) {
-                throw new InvalidSettingsDefinitionsException('Invalid default value for settings definition: ' . json_encode($options), 1732181102, null, $set['name'] ?? '');
+            try {
+                $this->settingDefinitionValidation->validate($definition);
+            } catch (InvalidSettingDefinitionException $e) {
+                throw new InvalidSettingsDefinitionsException(
+                    $e->getMessage(),
+                    1752483401,
+                    $e,
+                    $set['name'] ?? ''
+                );
             }
             $settingsDefinitions[] = $definition;
         }
