@@ -10,4 +10,4 @@
  *
  * The TYPO3 project - inspiring people to share!
  */
-class Severity{constructor(){this.loading=-3,this.notice=-2,this.info=-1,this.ok=0,this.warning=1,this.error=2}getCssClass(s){let e;switch(s){case this.loading:case this.notice:e="notice";break;case this.ok:e="success";break;case this.warning:e="warning";break;case this.error:e="danger";break;case this.info:default:e="info"}return e}}export default new Severity;
\ No newline at end of file
+class i{constructor(){this.loading=-3,this.notice=-2,this.info=-1,this.ok=0,this.warning=1,this.error=2}getCssClass(e){let s;switch(e){case this.loading:case this.notice:s="notice";break;case this.ok:s="success";break;case this.warning:s="warning";break;case this.error:s="danger";break;case this.info:default:s="info"}return s}}var t=new i;export{t as default};
